#include <JJX/JJX.hpp>
#include <iostream>
#include <fstream>

using namespace JJX;

std::string read_file(const std::string& file_path)
{
    std::ifstream file(file_path, std::ios::binary);
    if (!file)
        throw std::runtime_error("We couldn't find the file: " + file_path);

    std::streamsize file_size;
    file.seekg(0, std::ios::end);
    file_size = file.tellg();
    file.seekg(0, std::ios::beg);

    std::string file_content(file_size, '\0');
    file.read(&file_content[0], file_size);
    file.close();

    return file_content;
}


void parse_json_file(const std::string& file_path) {
    auto file_content = read_file(file_path);

    auto [text, parse_error] = json::parse(file_content);
    if (!parse_error.empty())
        std::cerr << "Error while parsing json: " << parse_error << std::endl;


    std::cout << json::deparse(text) << std::endl;
}

struct Vector3
{
    float x, y, z;
};


/// Creates json array out of Vector3 type.
json::value vector3_to_json(const Vector3& input)
{
    auto x = json::number(input.x);
    auto y = json::number(input.y);
    auto z = json::number(input.z);
    return json::array({x, y, z});
}

/// Creates Vector3 type out of json array object.
Vector3 json_to_vector3(const json::value& obj)
{
    Vector3 value;

    value.x = obj.array->at(0).number.value_or(0);
    value.y = obj.array->at(1).number.value_or(0);
    value.z = obj.array->at(2).number.value_or(0);

    return value;
}


struct product_info
{
    struct rating_metrics {
        double average;
        int total;
    };

    struct review
    {
        int rating;
        std::string review_text;
        std::string user;

        operator json::object_val() const
        {
            json::object_val obj = json::object();
            obj["rating"] = json::number(rating);
            obj["review_text"] = json::string(review_text);
            obj["user"] = json::string(user);
            return obj;
        }
    };

    std::string category;
    std::string description;
    bool is_available;
    std::string manufacturer;
    std::string name;
    double price;
    std::string product_id;
    rating_metrics ratings;
    std::string release_date;
    std::vector<review> reviews;
};


product_info pinfo_fromjson(json::value root)
{
    if (root.type != json::value_type::object)
        std::cerr << "Malformed product info file!" << std::endl;

    product_info this_product;

    json::object_val better_root = static_cast<json::object_val>(root);

    this_product.category = better_root["category"];
    this_product.description = better_root["description"];
    this_product.is_available = bool(better_root["is_available"]);
    this_product.manufacturer = better_root["manufacturer"];
    this_product.name = better_root["name"];
    this_product.price = double(better_root["price"]);

    json::array_val subobj = static_cast<json::array_val>(better_root["reviews"]);

    for (auto& review_data_pre : subobj.array.value())
    {
        json::object_val review_data = static_cast<json::object_val>(review_data_pre);
        product_info::review going_in;
        going_in.rating = double(review_data["rating"]);
        going_in.user = review_data["user"];
        going_in.review_text = review_data["review_text"];
        this_product.reviews.push_back(going_in);
    }

    return this_product;
}

json::value pinfo_tojson(product_info input)
{
    json::object_val root = json::object();
    root["category"] = json::string(input.category);
    root["manufacturer"] = json::string(input.manufacturer);
    root["is_available"] = input.is_available;
    //root.add("category", input.category);
    root.add("description", input.description);
    root["name"] = input.name;
    root["reviews"] = json::array();
    for (auto& rev : input.reviews) {
        root["reviews"].array.value().push_back(rev);
    }

    root["taters"] = 5.0;

    return root;
}


// Some progress has been made on small scale.
int main(int argc, char* argv[]) {
    //parse_json_file("../samples/product_info.json");
    auto file_contents = read_file("../samples/product_info.json");

    auto [text, err] = json::parse(file_contents);

    product_info test = pinfo_fromjson(text);


    auto result = pinfo_tojson(test);

    std::cout << json::deparse(result);
}
